from .src import EVENT_MAP, EVENTS, get_data_from_api, fix_opponents, normalize_date, merge_dicts, \
    get_extra_cols, SCHEDULE_COLS, RESULTS_COLS, IND_RESULTS_COLS, ROSTER_COLS
import pandas as pd
import numpy as np
from datetime import datetime


class RtnSingleTeamYear(object):
    def __init__(self, year, team_name, team_id=None):
        self.year = year
        if team_name is not None:
            self.team_name = team_name
            if team_id is None:
                self.team_id = self.get_team_id()
            else:
                self.team_id = team_id

    def get_team_mapping(self):
        all_teams_data = get_data_from_api(endpoint='gymnasts2', suffix=str(self.year) + '/1').json()
        return {team['team_name']: team['id'] for team in all_teams_data['teams']}

    def get_team_id(self):
        if not hasattr(self, 'team_id_map'):
            self.team_id_map = self.get_team_mapping()

        if self.team_name and self.team_name not in self.team_id_map.keys():
            raise ValueError(f'Unknown team name: {self.team_name}')

        return self.team_id_map.get(self.team_name, -1)
        # if self.team_name in self.team_id_map.keys():
        #     return self.team_id_map[self.team_name]
        # else:
        #     raise ValueError(f'{self.team_name} does not exist in data for {self.year}')

    def _get_raw_roster(self):
        rename_map = {'id': 'Gymnast ID', 'hometown': 'Hometown', 'school_year': 'School Year', 'events': 'Events'}
        school_year_map = {'1': 'FR', '2': 'SO', '3': 'JR', '4': 'SR'}

        roster_data = get_data_from_api(endpoint='rostermain', suffix=str(self.year)+'/'+str(self.team_id)+'/1').json()

        self._raw_roster = [{**{rename_map.get(k, k): v if k != 'school_year' else school_year_map.get(v, '') for k, v in data.items()},
                            **{'Name': data['fname'] + ' ' + data['lname'], 'Team': self.team_name}}
                            for data in roster_data]

    def get_roster(self, include_hometowns=False, include_class=False, include_events=False):
        if not hasattr(self, 'raw_roster'):
            self._get_raw_roster()

        extra_cols = get_extra_cols(include_hometowns=include_hometowns, include_class=include_class, include_events=include_events)

        if len(self._raw_roster) > 0:
            self.roster = pd.DataFrame([{k: v for k, v in data.items() if k in ROSTER_COLS + extra_cols}
                                        for data in self._raw_roster])
        else:
            self.roster = pd.DataFrame(columns=ROSTER_COLS + extra_cols)

        return self.roster

    def _get_raw_season_results(self):
        meets = get_data_from_api(endpoint='dashboard', suffix=str(self.year)+'/'+str(self.team_id)).json()
        name_map = {'team_id': 'Team ID', 'team_name': 'Team', 'meet_id': 'Team Meet ID',
                    'meet_date': 'Meet Date', 'team_score': 'Score', 'home': 'Home/Away',
                    'opponent': 'Opponents', 'meet_desc': 'Meet Name', 'linked_id': 'Meet ID'}

        self._raw_season_results = [{name_map.get(k, k): fix_opponents(v) if k == 'opponent'
                                    else (normalize_date(v) if k == 'meet_date' else v)
                                     for k, v in data.items() if k != 'jas'} for data in meets['meets'] if data['team_name'] == self.team_name]
        self._raw_schedule = [{k: v for k, v in data.items() if k not in ('Score', 'VT', 'UB', 'BB', 'FX')} for data in self._raw_season_results]

    def get_schedule(self):
        if not hasattr(self, '_raw_schedule'):
            self._get_raw_season_results()

        if len(self._raw_schedule) > 0:
            return pd.DataFrame(self._raw_schedule)
        else:
            return pd.DataFrame(columns=SCHEDULE_COLS)

    def get_team_scores(self, method='team_consistency', force_update=False):
        """
        Methods:
        * Team Consistency - uses the Team Consistency tab from RTN.
            * Only a single API call for all meets so much faster
            * Tends to have more complete data, especially for older years
            * Does not include meets (like event finals) where only individuals compete
            * Relies on date to join back to meet info, such as opponent, etc.
        * By Meet - loops through each meet to get scores
            * One API call per meet, so much slower
            * Older meets tend to be missing
            * Includes individual scores from meets where whole team did not compete
            * Uses team meet id to join back to meet info, such as opponent, etc.
        """
        if not hasattr(self, '_raw_season_results'):
            self._get_raw_season_results()

        if len(self._raw_season_results) > 0:
            if (len({'VT', 'UB', 'BB', 'FX'}.intersection(self._raw_season_results[0].keys())) != 4 or force_update):
                if method == 'team_consistency':
                    self._team_event_scores_team_consistency()
                elif method == 'by_meet':
                    self._team_event_scores_by_meet()
                else:
                    raise ValueError('Method must be "team_consistency" or "by_meet"')

            # TODO: different way to drop duplicates?
            self.season_results = pd.DataFrame(self._raw_season_results).dropna(subset=['Score']).drop_duplicates()
        else:
            self.season_results = pd.DataFrame(columns=SCHEDULE_COLS + RESULTS_COLS)

        return self.season_results

    def _team_event_scores_by_meet(self):
        team_scores_all = []
        for meet_id in [data['Team Meet ID'] for data in self._raw_season_results if data['Meet Date'] <= datetime.now()]:
            try:
                meet_res = get_data_from_api(endpoint='meetresults', suffix=str(meet_id)).json()
                # This API call returns scores from all teams at this meet, not just this team. Need to pick out correct score
                team_scores = [score for score in meet_res['teams'] if score['tname'] == self.team_name and score['mid'] == str(meet_id)]
                assert len(team_scores) == 1, 'Multiple team scores??'
                team_scores_all.append({EVENT_MAP.get(k, 'Team Meet ID'): v for k, v in team_scores[0].items() if k in ['mid', 'vault', 'bars', 'beam', 'floor']})
            except ValueError:
                print(f'Meet {meet_id} not found')

        if len(team_scores_all) > 0:
            merge_dicts(dict1=self._raw_season_results, dict2=team_scores_all, merge_field='Team Meet ID')
        else:
            print(f'No meet data found for year {self.year}')
            for i in range(len(self._raw_season_results)):
                self._raw_season_results[i].update({'VT': np.nan, 'UB': np.nan, 'BB': np.nan, 'FX': np.nan})

    def _team_event_scores_team_consistency(self):
        res = get_data_from_api(endpoint='teamConsistency', suffix=f'{self.year}/{self.team_id}').json()
        if len(res['labels']) == 0:
            print(f'No team consistency data found for {self.team_name} in {self.year}')
            for i in range(len(self._raw_season_results)):
                self._raw_season_results[i].update({'VT': np.nan, 'UB': np.nan, 'BB': np.nan, 'FX': np.nan})
        else:

            team_consistency = [{'Meet Date': normalize_date(res['labels'][i][:7] + str(self.year), dt_format='%b-%d-%Y'),
                                 'VT': round(float(res['vts'][i]), 4),
                                 'UB': round(float(res['ubs'][i]), 4),
                                 'BB': round(float(res['bbs'][i]), 4),
                                 'FX': round(float(res['fxs'][i]), 4)} for i in range(len(res['labels']))]

            merge_dicts(dict1=self._raw_season_results, dict2=team_consistency, merge_field='Meet Date')

    def get_individual_scores(self, method='individual_consistency', force_update=False):
        """
        Methods:
        * Individual Consistency - Uses Individual Consistency tab from RTN
            * Tends to have more complete data, especially for older years
            * Relies on date to join back to meet info, such as opponent, etc.
            * One API call per gymnast, relative speed depends on number of meets vs number of gymnasts
        * By Meet - loops through each meet to get scores
            * Older meets tend to be missing
            * Uses team meet id to join back to meet info, such as opponent, etc.
            * One API call per meet, relative speed depends on number of meets vs number of gymnasts
        """
        if not hasattr(self, '_raw_schedule'):
            self.get_schedule()

        if force_update or not hasattr(self, 'individual_results'):
            if method == 'individual_consistency':
                if not hasattr(self, '_raw_roster'):
                    self.get_roster()

                self._individual_scores_individual_consistency()
            elif method == 'by_meet':
                self._individual_scores_by_meet()
            else:
                raise ValueError('Method must be "individual_consistency" or "by_meet"')

        return self.individual_results

    def _individual_scores_by_meet(self):
        individual_scores_all = []
        for meet_id in [meet['Team Meet ID'] for meet in self._raw_schedule if meet['Meet Date'] <= datetime.now()]:
            try:
                meet_res = get_data_from_api(endpoint='meetresults', suffix=str(meet_id)).json()
                if len(meet_res) == 0 or len(meet_res['scores']) == 0 or len(meet_res['scores'][0]) == 0:
                    print(f'No data found for meet {meet_id}')
                    continue

                if 'team_name' in meet_res['scores'][0][0]:
                    team_inds = [ind for ind, scores in enumerate(meet_res['scores']) if len(scores) > 0 and scores[0]['team_name'] == self.team_name]
                else:
                    raise ValueError('Key not found')

                if len(team_inds) == 0:
                    print(f'No scores found at meet {meet_id}')
                    continue
                team_ind = team_inds[0]

                individual_scores = [{**{EVENT_MAP.get(k, {'gid': 'Gymnast ID'}.get(k, k)):
                                         v if k not in EVENT_MAP.keys() else (round(float(v), 4) if v is not None else np.nan)
                                         for k, v in data.items() if k in ['gid'] + list(EVENT_MAP.keys())},
                                      **{'Team Meet ID': str(meet_id),
                                         'Name': data['first_name'] + ' ' + data['last_name']}}
                                     for data in meet_res['scores'][team_ind]]

                individual_scores_all.extend(individual_scores)
            except ValueError:
                print(f'Meet {meet_id} not found')

        if len(individual_scores_all) > 0:
            merge_dicts(dict1=individual_scores_all, dict2=self._raw_schedule, merge_field='Team Meet ID')
            self.individual_results = pd.DataFrame(individual_scores_all)
            self.individual_results['AA'] = self.individual_results[['VT', 'UB', 'BB', 'FX']].dropna(how='any').astype(float).T.sum().round(4)
        else:
            self.individual_results = pd.DataFrame(columns=['Meet Date', 'VT', 'UB', 'BB', 'FX', 'AA', 'Gymnast ID', 'Name',
                                                    'Team ID', 'Team', 'Team Meet ID', 'Home/Away', 'Opponents',
                                                    'Meet Name', 'Meet ID'])

    def _individual_scores_individual_consistency(self):
        ind_consistency_all = []
        for gymnast in self._raw_roster:
            try:
                res = get_data_from_api(endpoint='indConsistency', suffix=f"{self.year}/{gymnast['Gymnast ID']}").json()
                ind_consistency = [{'Meet Date': normalize_date(res['labels'][i][:7] + str(self.year), dt_format='%b-%d-%Y'),
                                    'VT': round(float(res['vts'][i]), 4) if res['vts'][i] is not None else np.nan,
                                    'UB': round(float(res['ubs'][i]), 4) if res['ubs'][i] is not None else np.nan,
                                    'BB': round(float(res['bbs'][i]), 4) if res['bbs'][i] is not None else np.nan,
                                    'FX': round(float(res['fxs'][i]), 4) if res['fxs'][i] is not None else np.nan,
                                    'AA': round(float(res['vts'][i]) + float(res['ubs'][i]) + float(res['bbs'][i]) + float(res['fxs'][i]), 4) if all([res['vts'][i],res['ubs'][i],res['bbs'][i],res['fxs'][i]]) else np.nan,
                                    'Gymnast ID': gymnast['Gymnast ID'],
                                    'Name': gymnast['Name']
                                    } for i in range(len(res['labels']))]
                ind_consistency_all.extend(ind_consistency)

            except ValueError:
                print(f"No individual consistency scores found for {gymnast['Name']}")

        if len(ind_consistency_all) > 0:
            merge_dicts(dict1=ind_consistency_all, dict2=self._raw_schedule, merge_field='Meet Date')
            self.individual_results = pd.DataFrame(ind_consistency_all)
        else:
            self.individual_results = pd.DataFrame(columns=SCHEDULE_COLS + IND_RESULTS_COLS)

    def get_individual_nqs(self):
        if not hasattr(self, '_raw_roster'):
            self._get_raw_roster()

        if not hasattr(self, '_raw_individual_nqs'):
            self._get_raw_individual_nqs()

        if len(self._raw_individual_nqs) > 0:
            return pd.DataFrame(self._raw_individual_nqs)
        else:
            return pd.DataFrame(columns=ROSTER_COLS + EVENTS) # + ['AA'])

    def _get_raw_individual_nqs(self):
        name_map = {'maxv': 'VT', 'maxub': 'UB', 'maxbb': 'BB', 'maxfx': 'FX',
                    # 'maxaa': 'AA',
                    'gid': 'Gymnast ID'}

        nqsData = get_data_from_api(endpoint='rostermain', suffix=f'{self.year}/{self.team_id}/4').json()
        ind_nqs = [{name_map[k]: round(float(v), 4) if k != 'gid' and v != '' else (np.nan if k != 'gid' else v)
                    for k, v in data.items() if k in name_map.keys()} for data in nqsData['ind']]

        if len(ind_nqs) > 0:
            self._raw_individual_nqs = [{k: v for k, v in data.items() if k in ['Gymnast ID', 'Name', 'Team']}
                                        for data in self._raw_roster]
            merge_dicts(dict1=self._raw_individual_nqs, dict2=ind_nqs, merge_field='Gymnast ID')
        else:
            self._raw_individual_nqs = []

    def _get_current_week(self):
        if not hasattr(self, 'week'):
            week_data = get_data_from_api(endpoint='currentweek', suffix=str(self.year)).json()
            return min(int(week_data['week']), int(week_data['max']))

    def _get_raw_rankings(self, team_vs_ind, event, week):
        team_ind_map = {'team': 0, 'ind': 1}
        event_api_map = {'VT': 1, 'UB': 2, 'BB': 3, 'FX': 4, 'AA': 5}
        rename_map = {'rank': 'Rank', 'gid': 'Gymnast ID', 'team': 'Team', 'tid': 'Team ID',
                      'rqs': 'NQS', 'reg': 'Region', 'con': 'Conference', 'div': 'Division',
                      'usag': 'USAG', 'ave': 'Average', 'high': 'High', 'name': 'Team'}

        res = get_data_from_api(endpoint='results', suffix=f'{self.year}/{week}/{team_ind_map[team_vs_ind]}/{event_api_map[event]}').json()
        if team_vs_ind == 'ind':
            self._raw_rankings[team_vs_ind][event] = [{**{rename_map.get(k): float(v) if k in ['rqs', 'ave', 'high'] else v for k, v in data.items() if k in rename_map},
                                                       **{'Name': data['fname'] + ' ' + data['lname'], 'Event': event}}
                                                      for data in res['data']]
        else:
            self._raw_rankings[team_vs_ind][event] = [{**{rename_map.get(k): float(v) if k in ['rqs', 'ave', 'high'] else v for k, v in data.items() if k in rename_map},
                                                       **{'Event': event}}
                                                      for data in res['data']]

    def get_overall_rankings(self, team_vs_ind='team', event='AA', week=None):
        if not week:
            week = self._get_current_week()

        if not hasattr(self, '_raw_rankings'):
            self._raw_rankings = {'team': {event: None for event in EVENT_MAP.values()},
                                  'ind': {event: None for event in EVENT_MAP.values()}}

        col_orders = {'ind': ['Event', 'Rank', 'Gymnast ID', 'Name', 'Team ID', 'Team', 'NQS', 'Average', 'High',
                              'Division', 'Conference', 'Region', 'USAG'],
                      'team': ['Event', 'Rank', 'Team ID', 'Team', 'NQS', 'Average', 'High',
                               'Division', 'Conference', 'Region', 'USAG']}

        if self._raw_rankings[team_vs_ind][event] is None:
            self._get_raw_rankings(team_vs_ind=team_vs_ind, event=event, week=week)

        return pd.DataFrame(self._raw_rankings[team_vs_ind][event])[col_orders[team_vs_ind]]





import requests
from requests.adapters import HTTPAdapter
from urllib3.util.retry import Retry
from functools import lru_cache
from datetime import datetime

EVENT_MAP = {'vault': 'VT', 'bars': 'UB', 'beam': 'BB', 'floor': 'FX', 'all_around': 'AA'}
EVENTS = ['VT', 'UB', 'BB', 'FX']
ROSTER_COLS = ['Gymnast ID', 'Name', 'Team']
SCHEDULE_COLS = ['Team ID', 'Team', 'Team Meet ID', 'Meet Date', 'Home/Away', 'Opponents', 'Meet Name', 'Meet ID']
RESULTS_COLS = ['Score'] + EVENTS
IND_RESULTS_COLS = ['Gymnast ID', 'Name'] + EVENTS + ['AA']


def validate_input(teams):
    if isinstance(teams, str):
        teams = [teams]
    elif not isinstance(teams, list):
        raise TypeError('teams must be a string or a list')

    return teams


@lru_cache(maxsize=1000000)
def get_data_from_api(endpoint, suffix):
    session = requests.Session()
    retry = Retry(connect=3, backoff_factor=0.5)
    adapter = HTTPAdapter(max_retries=retry)
    session.mount('http://', adapter)
    session.mount('https://', adapter)

    url = 'https://www.roadtonationals.com/api/women/' + endpoint
    if suffix:
        url += '/' + suffix

    return session.get(url)


def fix_opponents(ops):
    if not isinstance(ops, str):
        return ops

    ops = (ops.replace(', ', '/').replace(',','/').replace(' and ', '/').replace(' @ ', '/').replace(' w/ ', '/').replace(' with ', '/'))

    if 'william & mary' in ops.lower():
        # Todo: title case is going to mess with other opponents here, full mapping might fix that
        ops = ops.title().replace('William & Mary', 'William&Mary').replace(' & ', '/').replace('William&Mary', 'William & Mary')
    else:
        ops = ops.replace(' & ', '/')

    if ops[:3] == 'at ':
        ops = ops[3:]
    if '(' in ops and ops[-1] == ')':
        ops = ops.split('(')[0].strip()

    ops = '/'.join([OP_MAP.get(op.strip(), OP_MAP.get(op.lower(), op)) for op in ops.split('/')])
    return ops


def normalize_date(dt, dt_format='%a, %b-%d-%Y'):
    return datetime.strptime(dt, dt_format)


def merge_dicts(dict1, dict2, merge_field):
    for i in range(len(dict1)):
        # they're likely in the same order
        try:
            if dict1[i][merge_field] == dict2[i][merge_field]:
                dict1[i].update(dict2[i])
                continue
        except IndexError:
            pass

        # if not in the same order, search through the list to find the one with the same date
        for j in range(len(dict2)):
            if dict1[i][merge_field] == dict2[j][merge_field]:
                dict1[i].update(dict2[j])
                break


def get_extra_cols(include_hometowns, include_class, include_events):
    extra_cols = []
    if include_hometowns:
        extra_cols.append('Hometown')
    if include_class:
        extra_cols.append('School Year')
    if include_events:
        extra_cols.append('Events')

    return extra_cols


OP_MAP = {
    "AIR FORCE ACADEMY": "Air Force",
    "AFA": "Air Force",
    "air force academy": "Air Force",
    "Air Force Academy": "Air Force",
    "AIR FORCE": "Air Force",
    "afa": "Air Force",
    "Air Force": "Air Force",
    "Afa": "Air Force",
    "air force": "Air Force",
    "alabama": "Alabama",
    "Alabama": "Alabama",
    "ALABAMA": "Alabama",
    "uaa": "Alaska",
    "Uaa": "Alaska",
    "ALASKA": "Alaska",
    "Alaska": "Alaska",
    "UAA": "Alaska",
    "alaska": "Alaska",
    "arizona": "Arizona",
    "ARIZONA": "Arizona",
    "Arizona": "Arizona",
    "asu": "Arizona State",
    "Asu": "Arizona State",
    "arizona state": "Arizona State",
    "Arizona State": "Arizona State",
    "ASU": "Arizona State",
    "ARIZONA STATE": "Arizona State",
    "ARKANSAS": "Arkansas",
    "Arkansas": "Arkansas",
    "arkansas": "Arkansas",
    "AUBURN": "Auburn",
    "Auburn": "Auburn",
    "auburn": "Auburn",
    "BRIGHAM YOUNG": "BYU",
    "Brigham Young": "BYU",
    "BYU": "BYU",
    "brigham young": "BYU",
    "byu": "BYU",
    "Byu": "BYU",
    "BALL STATE": "Ball State",
    "ball state": "Ball State",
    "BALL STATE UNIVERSITY": "Ball State",
    "ball state university": "Ball State",
    "Ball State": "Ball State",
    "Ball State University": "Ball State",
    "boise": "Boise State",
    "boise state": "Boise State",
    "BOISE": "Boise State",
    "BOISE STATE": "Boise State",
    "Boise": "Boise State",
    "Boise State": "Boise State",
    "Bgsu": "Bowling Green",
    "bgsu": "Bowling Green",
    "Bowling Green": "Bowling Green",
    "BGSU": "Bowling Green",
    "bowling green": "Bowling Green",
    "BOWLING GREEN": "Bowling Green",
    "bowling green state": "Bowling Green",
    "Bowling Green State": "Bowling Green",
    "BOWLING GREEN STATE": "Bowling Green",
    "BRIDGEPORT": "Bridgeport",
    "bridgeport": "Bridgeport",
    "Bridgeport": "Bridgeport",
    "Brockport": "Brockport",
    "brockport": "Brockport",
    "BROCKPORT": "Brockport",
    "BROWN": "Brown",
    "Brown": "Brown",
    "brown": "Brown",
    "cal state fullerton": "Cal State Fullerton",
    "CS Fullerton": "Cal State Fullerton",
    "CSUF": "Cal State Fullerton",
    "Cal State-Fullerton": "Cal State Fullerton",
    "CAL STATE-FULLERTON": "Cal State Fullerton",
    "Csuf": "Cal State Fullerton",
    "Fullerton": "Cal State Fullerton",
    "csuf": "Cal State Fullerton",
    "fullerton": "Cal State Fullerton",
    "cs fullerton": "Cal State Fullerton",
    "cal state-fullerton": "Cal State Fullerton",
    "Cs Fullerton": "Cal State Fullerton",
    "Cal State Fullerton": "Cal State Fullerton",
    "FULLERTON": "Cal State Fullerton",
    "CS FULLERTON": "Cal State Fullerton",
    "CAL STATE FULLERTON": "Cal State Fullerton",
    "Cal Berkley": "California",
    "CALIFORNIA": "California",
    "California": "California",
    "cal": "California",
    "UC Berkely": "California",
    "california": "California",
    "UC BERKELY": "California",
    "cal berkley": "California",
    "Cal": "California",
    "CAL": "California",
    "uc berkely": "California",
    "Uc Berkely": "California",
    "CAL BERKLEY": "California",
    "Centenary College": "Centenary",
    "CENTENARY COLLEGE": "Centenary",
    "centenary": "Centenary",
    "centenary college": "Centenary",
    "CENTENARY": "Centenary",
    "Centenary": "Centenary",
    "CMU": "Central Michigan",
    "cmu": "Central Michigan",
    "Central Michigan": "Central Michigan",
    "Cmu": "Central Michigan",
    "central michigan": "Central Michigan",
    "CENTRAL MICHIGAN": "Central Michigan",
    "Cornell": "Cornell",
    "cornell": "Cornell",
    "CORNELL": "Cornell",
    "CORTLAND": "Cortland",
    "cortland": "Cortland",
    "cortland state": "Cortland",
    "CORTLAND STATE": "Cortland",
    "Cortland State": "Cortland",
    "Cortland": "Cortland",
    "denver": "Denver",
    "Denver": "Denver",
    "DENVER": "Denver",
    "E. MICHIGAN UNIV": "Eastern Michigan",
    "e. michigan univ": "Eastern Michigan",
    "Eastern Michigan": "Eastern Michigan",
    "emu": "Eastern Michigan",
    "EMU": "Eastern Michigan",
    "EASTERN MICHIGAN": "Eastern Michigan",
    "eastern michigan": "Eastern Michigan",
    "Emu": "Eastern Michigan",
    "E. Michigan Univ": "Eastern Michigan",
    "FISK": "Fisk",
    "Fisk": "Fisk",
    "fisk": "Fisk",
    "FLORIDA": "Florida",
    "florida": "Florida",
    "Florida": "Florida",
    "George Washington": "George Washington",
    "GEORGE WASHINGTON UNIVERSITY": "George Washington",
    "Gwu": "George Washington",
    "george washington university": "George Washington",
    "george washington": "George Washington",
    "gwu": "George Washington",
    "George Washington University": "George Washington",
    "GEORGE WASHINGTON": "George Washington",
    "GWU": "George Washington",
    "Georgia": "Georgia",
    "GEORGIA": "Georgia",
    "georgia": "Georgia",
    "GREENVILLE": "Greenville",
    "Greenville": "Greenville",
    "greenville": "Greenville",
    "Gustavus Adolphus": "Gustavus Adolphus",
    "gustavus adolphus": "Gustavus Adolphus",
    "GUSTAVUS ADOLPHUS": "Gustavus Adolphus",
    "Hamline": "Hamline",
    "HAMLINE": "Hamline",
    "hamline": "Hamline",
    "ILLINOIS": "Illinois",
    "illinois": "Illinois",
    "university of illinois": "Illinois",
    "UNIVERSITY OF ILLINOIS": "Illinois",
    "Univ Of Ill": "Illinois",
    "University of Illinois": "Illinois",
    "Illinois": "Illinois",
    "University Of Illinois": "Illinois",
    "UNIV OF ILL": "Illinois",
    "univ of ill": "Illinois",
    "illinois state": "Illinois State",
    "ILLINOIS STATE": "Illinois State",
    "Illinois State": "Illinois State",
    "Indiana": "Indiana",
    "indiana": "Indiana",
    "INDIANA": "Indiana",
    "University Of Iowa": "Iowa",
    "iowa": "Iowa",
    "University of Iowa": "Iowa",
    "Iowa": "Iowa",
    "university of iowa": "Iowa",
    "UNIVERSITY OF IOWA": "Iowa",
    "IOWA": "Iowa",
    "iowa state": "Iowa State",
    "IOWA STATE": "Iowa State",
    "Iowa State": "Iowa State",
    "Ithaca College": "Ithaca College",
    "ITHACA COLLEGE": "Ithaca College",
    "ithaca college": "Ithaca College",
    "James Madison Univ": "James Madison",
    "james madison university": "James Madison",
    "james madison univ": "James Madison",
    "JAMES MADISON UNIV": "James Madison",
    "james madison u": "James Madison",
    "JMU": "James Madison",
    "James Madison": "James Madison",
    "jmu": "James Madison",
    "JAMES MADISON": "James Madison",
    "JAMES MADISON U": "James Madison",
    "Jmu": "James Madison",
    "james madison": "James Madison",
    "James Madison University": "James Madison",
    "JAMES MADISON UNIVERSITY": "James Madison",
    "James Madison U": "James Madison",
    "Kent State": "Kent State",
    "Kent": "Kent State",
    "kent": "Kent State",
    "KENT STATE": "Kent State",
    "KENT": "Kent State",
    "kent state": "Kent State",
    "Uk": "Kentucky",
    "kentucky": "Kentucky",
    "uk": "Kentucky",
    "UK": "Kentucky",
    "Kentucky": "Kentucky",
    "KENTUCKY": "Kentucky",
    "Liu": "LIU",
    "LIU": "LIU",
    "liu": "LIU",
    "lsu": "LSU",
    "LSU": "LSU",
    "Lsu": "LSU",
    "LINDENWOOD": "Lindenwood",
    "Lindenwood": "Lindenwood",
    "lindenwood": "Lindenwood",
    "Maryland": "Maryland",
    "MARYLAND": "Maryland",
    "maryland": "Maryland",
    "U Massachusetts": "Massachusetts",
    "Massachusetts": "Massachusetts",
    "massachusetts": "Massachusetts",
    "U MASSACHUSETTS": "Massachusetts",
    "umass": "Massachusetts",
    "Umass": "Massachusetts",
    "u massachusetts": "Massachusetts",
    "UMASS": "Massachusetts",
    "MASSACHUSETTS": "Massachusetts",
    "UMass": "Massachusetts",
    "MICHIGAN": "Michigan",
    "Michigan": "Michigan",
    "michigan": "Michigan",
    "MICHIGAN STATE": "Michigan State",
    "mich st": "Michigan State",
    "Michigan State": "Michigan State",
    "MICH ST": "Michigan State",
    "Mich St": "Michigan State",
    "msu": "Michigan State",
    "michigan state": "Michigan State",
    "MSU": "Michigan State",
    "Msu": "Michigan State",
    "Minnesota": "Minnesota",
    "MINNESOTA": "Minnesota",
    "minnesota": "Minnesota",
    "MISSOURI": "Missouri",
    "Missouri": "Missouri",
    "missouri": "Missouri",
    "nc state": "N.C. State",
    "NC STATE": "N.C. State",
    "North Carolina State": "N.C. State",
    "NC State University": "N.C. State",
    "north carolina state": "N.C. State",
    "NORTH CAROLINA STATE": "N.C. State",
    "Nc State University": "N.C. State",
    "nc state university": "N.C. State",
    "n.c. state": "N.C. State",
    "N.C. STATE": "N.C. State",
    "Nc State": "N.C. State",
    "NC STATE UNIVERSITY": "N.C. State",
    "NC State": "N.C. State",
    "N.C. State": "N.C. State",
    "Nebraska": "Nebraska",
    "nebraska": "Nebraska",
    "NEBRASKA": "Nebraska",
    "NEW HAMPSHIRE": "New Hampshire",
    "unh": "New Hampshire",
    "UNH": "New Hampshire",
    "new hampshire": "New Hampshire",
    "Unh": "New Hampshire",
    "New Hampshire": "New Hampshire",
    "new mexico": "New Mexico",
    "New Mexico": "New Mexico",
    "NEW MEXICO": "New Mexico",
    "north carolina": "North Carolina",
    "University of North Carolina": "North Carolina",
    "UNIVERSITY OF NORTH CAROLINA": "North Carolina",
    "Unc": "North Carolina",
    "North Carolina": "North Carolina",
    "UNC": "North Carolina",
    "University Of North Carolina": "North Carolina",
    "university of north carolina": "North Carolina",
    "NORTH CAROLINA": "North Carolina",
    "unc": "North Carolina",
    "N Illinois Univ": "Northern Illinois",
    "NORTHERN ILLINOIS UNIVERSITY": "Northern Illinois",
    "N ILLINOIS UNIV": "Northern Illinois",
    "northern illinois": "Northern Illinois",
    "NIU": "Northern Illinois",
    "NORTHERN ILLINOIS": "Northern Illinois",
    "Niu": "Northern Illinois",
    "northern illinois university": "Northern Illinois",
    "Northern Illinois University": "Northern Illinois",
    "Northern Illinois": "Northern Illinois",
    "niu": "Northern Illinois",
    "n illinois univ": "Northern Illinois",
    "ohio state": "Ohio State",
    "Ohio State": "Ohio State",
    "OHIO STATE": "Ohio State",
    "oklahoma": "Oklahoma",
    "Oklahoma": "Oklahoma",
    "OKLAHOMA": "Oklahoma",
    "OREGON STATE": "Oregon State",
    "oregon state": "Oregon State",
    "OSU": "Oregon State",
    "Osu": "Oregon State",
    "osu": "Oregon State",
    "Oregon State": "Oregon State",
    "upenn": "Penn",
    "Penn": "Penn",
    "PENN": "Penn",
    "Pennsylvania": "Penn",
    "penn": "Penn",
    "pennsylvania": "Penn",
    "PENNSYLVANIA": "Penn",
    "UPenn": "Penn",
    "UPENN": "Penn",
    "Upenn": "Penn",
    "psu": "Penn State",
    "Psu": "Penn State",
    "penn state": "Penn State",
    "Penn St": "Penn State",
    "PENN ST": "Penn State",
    "penn st": "Penn State",
    "PENN STATE": "Penn State",
    "Penn State": "Penn State",
    "PSU": "Penn State",
    "PITT": "Pittsburgh",
    "University Of Pittsburgh": "Pittsburgh",
    "University of Pittsburgh": "Pittsburgh",
    "UNIVERSITY OF PITTSBURGH": "Pittsburgh",
    "pittsburgh": "Pittsburgh",
    "Pittsburgh": "Pittsburgh",
    "Pitt": "Pittsburgh",
    "university of pittsburgh": "Pittsburgh",
    "PITTSBURGH": "Pittsburgh",
    "pitt": "Pittsburgh",
    "radford": "Radford",
    "Radford": "Radford",
    "RADFORD": "Radford",
    "Rhode Island College": "Rhode Island College",
    "Ric": "Rhode Island College",
    "rhode island college": "Rhode Island College",
    "ric": "Rhode Island College",
    "RHODE ISLAND COLLEGE": "Rhode Island College",
    "RIC": "Rhode Island College",
    "RUTGERS UNIVERSITY": "Rutgers",
    "RUTGERS": "Rutgers",
    "Rutgers University": "Rutgers",
    "rutgers": "Rutgers",
    "rutgers university": "Rutgers",
    "Rutgers": "Rutgers",
    "SEMO": "SEMO",
    "S.E. Missouri": "SEMO",
    "S.E. MISSOURI": "SEMO",
    "Semo": "SEMO",
    "Se Missouri St.": "SEMO",
    "Southeast Missouri": "SEMO",
    "southeast missouri": "SEMO",
    "Se Missouri St": "SEMO",
    "SE Missouri St.": "SEMO",
    "southeast missouri state": "SEMO",
    "se missouri st": "SEMO",
    "SE MISSOURI ST.": "SEMO",
    "SOUTHEAST MISSOURI": "SEMO",
    "se missouri st.": "SEMO",
    "SEMO State": "SEMO",
    "s.e. missouri": "SEMO",
    "SEMO STATE": "SEMO",
    "semo state": "SEMO",
    "SOUTHEAST MISSOURI STATE": "SEMO",
    "Southeast Missouri State": "SEMO",
    "SE MISSOURI ST": "SEMO",
    "SE Missouri ST": "SEMO",
    "semo": "SEMO",
    "Semo State": "SEMO",
    "SAC STATE": "Sacramento State",
    "Sacramento State": "Sacramento State",
    "sac state": "Sacramento State",
    "Sac St": "Sacramento State",
    "Sac State": "Sacramento State",
    "SACRAMENTO STATE": "Sacramento State",
    "sac st": "Sacramento State",
    "SAC ST": "Sacramento State",
    "csus": "Sacramento State",
    "CSUS": "Sacramento State",
    "sacramento state": "Sacramento State",
    "Csus": "Sacramento State",
    "san jose": "San Jose State",
    "SAN JOSE STATE": "San Jose State",
    "San Jose State": "San Jose State",
    "San Jose": "San Jose State",
    "SAN JOSE": "San Jose State",
    "san jose state": "San Jose State",
    "seattle pacific": "Seattle Pacific",
    "spu": "Seattle Pacific",
    "Spu": "Seattle Pacific",
    "Seattle Pacific": "Seattle Pacific",
    "SPU": "Seattle Pacific",
    "SEATTLE PACIFIC": "Seattle Pacific",
    "simpson": "Simpson",
    "SIMPSON": "Simpson",
    "Simpson": "Simpson",
    "SCSU": "Southern Connecticut",
    "Southern CT": "Southern Connecticut",
    "SOUTHERN CONN UNIVERSITY": "Southern Connecticut",
    "southern conn university": "Southern Connecticut",
    "scsu": "Southern Connecticut",
    "Scsu": "Southern Connecticut",
    "SOUTHERN CT": "Southern Connecticut",
    "Southern Conn University": "Southern Connecticut",
    "Southern Ct": "Southern Connecticut",
    "southern ct": "Southern Connecticut",
    "SOUTHERN CONNECTICUT": "Southern Connecticut",
    "Southern Conn.": "Southern Connecticut",
    "Southern Connecticut": "Southern Connecticut",
    "southern conn.": "Southern Connecticut",
    "southern connecticut": "Southern Connecticut",
    "SOUTHERN CONN.": "Southern Connecticut",
    "southern illinois university": "Southern Illinois University",
    "Siu": "Southern Illinois University",
    "siu": "Southern Illinois University",
    "SIU": "Southern Illinois University",
    "Southern Illinois University": "Southern Illinois University",
    "SOUTHERN ILLINOIS UNIVERSITY": "Southern Illinois University",
    "Southern Utah": "Southern Utah",
    "S UTAH": "Southern Utah",
    "s utah": "Southern Utah",
    "so utah": "Southern Utah",
    "SO UTAH": "Southern Utah",
    "Suu": "Southern Utah",
    "SOUTHERN UTAH": "Southern Utah",
    "southern utah": "Southern Utah",
    "SOUTHERN UTAH UNIV.": "Southern Utah",
    "SUU": "Southern Utah",
    "suu": "Southern Utah",
    "So Utah": "Southern Utah",
    "Southern Utah Univ.": "Southern Utah",
    "S Utah": "Southern Utah",
    "southern utah univ.": "Southern Utah",
    "springfield": "Springfield",
    "SPRINGFIELD": "Springfield",
    "Springfield": "Springfield",
    "springfield college": "Springfield",
    "Springfield College": "Springfield",
    "SPRINGFIELD COLLEGE": "Springfield",
    "stanford": "Stanford",
    "STANFORD": "Stanford",
    "Stanford": "Stanford",
    "temple": "Temple",
    "TEMPLE": "Temple",
    "Temple": "Temple",
    "Tx Woman'S Univ.": "Texas Woman's",
    "TX WOMAN'S": "Texas Woman's",
    "TX Woman's Univ.": "Texas Woman's",
    "TWU": "Texas Woman's",
    "texas woman's univ": "Texas Woman's",
    "TX. WOMAN'S UNIV.": "Texas Woman's",
    "Texas Woman'S": "Texas Woman's",
    "TEXAS WOMAN'S UNIV": "Texas Woman's",
    "tx woman's": "Texas Woman's",
    "Texas Woman'S Univ": "Texas Woman's",
    "TEXAS WOMAN'S": "Texas Woman's",
    "Texas Women's": "Texas Woman's",
    "tx woman's univ.": "Texas Woman's",
    "texas woman's": "Texas Woman's",
    "Texas Women'S": "Texas Woman's",
    "TX WOMAN'S UNIV.": "Texas Woman's",
    "TX. Woman's Univ.": "Texas Woman's",
    "Texas Woman's": "Texas Woman's",
    "Tx Woman'S": "Texas Woman's",
    "texas women's": "Texas Woman's",
    "Tx. Woman'S Univ.": "Texas Woman's",
    "TX Woman's": "Texas Woman's",
    "twu": "Texas Woman's",
    "Twu": "Texas Woman's",
    "tx. woman's univ.": "Texas Woman's",
    "TEXAS WOMEN'S": "Texas Woman's",
    "Towson University": "Towson",
    "TOWSON STATE": "Towson",
    "Towson": "Towson",
    "TOWSON UNIVERSITY": "Towson",
    "towson university": "Towson",
    "Towson State": "Towson",
    "TOWSON": "Towson",
    "towson state": "Towson",
    "towson": "Towson",
    "Davis": "UC Davis",
    "UC Davis": "UC Davis",
    "UC DAVIS": "UC Davis",
    "uc davis": "UC Davis",
    "davis": "UC Davis",
    "DAVIS": "UC Davis",
    "Uc Davis": "UC Davis",
    "ucsb": "UC-Santa Barbara",
    "Uc-Santa Barbara": "UC-Santa Barbara",
    "uc-santa barbara": "UC-Santa Barbara",
    "UC SANTA BARBARA": "UC-Santa Barbara",
    "UC-SANTA BARBARA": "UC-Santa Barbara",
    "Ucsb": "UC-Santa Barbara",
    "UC Santa Barbara": "UC-Santa Barbara",
    "UC-Santa Barbara": "UC-Santa Barbara",
    "uc santa barbara": "UC-Santa Barbara",
    "Uc Santa Barbara": "UC-Santa Barbara",
    "UCSB": "UC-Santa Barbara",
    "ucla": "UCLA",
    "UCLA": "UCLA",
    "la": "UCLA",
    "La": "UCLA",
    "LA": "UCLA",
    "Ucla": "UCLA",
    "illinois-chicago": "UIC",
    "Illinios-Chicago": "UIC",
    "uic": "UIC",
    "Illinois-Chicago": "UIC",
    "Uic": "UIC",
    "illinios-chicago": "UIC",
    "UIC": "UIC",
    "ILLINOIS-CHICAGO": "UIC",
    "ILLINIOS-CHICAGO": "UIC",
    "univ. of rhode island": "URI",
    "UNIVERSITY OF RHODE ISLAND": "URI",
    "URI": "URI",
    "uri": "URI",
    "Univof Rhode Island": "URI",
    "UNIVOF RHODE ISLAND": "URI",
    "univof rhode island": "URI",
    "University of Rhode Island": "URI",
    "UNIV. OF RHODE ISLAND": "URI",
    "University Of Rhode Island": "URI",
    "university of rhode island": "URI",
    "Uri": "URI",
    "Univ. Of Rhode Island": "URI",
    "Univ. of Rhode Island": "URI",
    "UW-EAU CLAIRE": "UW-Eau Claire",
    "Uw-Eau Claire": "UW-Eau Claire",
    "uw-eau claire": "UW-Eau Claire",
    "UW-Eau Claire": "UW-Eau Claire",
    "uw-la crosse": "UW-La Crosse",
    "wisconsin-lacrosse": "UW-La Crosse",
    "UW-LA CROSSE": "UW-La Crosse",
    "Uw-La Crosse": "UW-La Crosse",
    "UW-La Crosse": "UW-La Crosse",
    "WISCONSIN-LACROSSE": "UW-La Crosse",
    "Wisconsin-Lacrosse": "UW-La Crosse",
    "Uw-Oshkosh": "UW-Oshkosh",
    "uw-oshkosh": "UW-Oshkosh",
    "UW-Oshkosh": "UW-Oshkosh",
    "OSHKOSH": "UW-Oshkosh",
    "UW-OSHKOSH": "UW-Oshkosh",
    "oshkosh": "UW-Oshkosh",
    "Oshkosh": "UW-Oshkosh",
    "Uw-Stout": "UW-Stout",
    "UW-STOUT": "UW-Stout",
    "UW-Stout": "UW-Stout",
    "uw-stout": "UW-Stout",
    "UW-WHITEWATER": "UW-Whitewater",
    "Uw-Whitewater": "UW-Whitewater",
    "UW-Whitewater": "UW-Whitewater",
    "uw-whitewater": "UW-Whitewater",
    "URSINUS COLLEGE": "Ursinus",
    "ursinus college": "Ursinus",
    "URSINUS": "Ursinus",
    "Ursinus": "Ursinus",
    "ursinus": "Ursinus",
    "Ursinus College": "Ursinus",
    "utah": "Utah",
    "UTAH": "Utah",
    "Utah": "Utah",
    "utah state": "Utah State",
    "UTAH STATE": "Utah State",
    "Utah State": "Utah State",
    "University Of Vermont": "Vermont",
    "vermont": "Vermont",
    "Vermont": "Vermont",
    "university of vermont": "Vermont",
    "VERMONT": "Vermont",
    "UNIVERSITY OF VERMONT": "Vermont",
    "University of Vermont": "Vermont",
    "Washington": "Washington",
    "washington": "Washington",
    "WASHINGTON": "Washington",
    "WEST CHESTER": "West Chester",
    "west chester": "West Chester",
    "West Chester": "West Chester",
    "West Virginia Univ.": "West Virginia",
    "West Virginia": "West Virginia",
    "WEST VIRGINIA": "West Virginia",
    "WVU": "West Virginia",
    "west virginia": "West Virginia",
    "WEST VIRGINIA UNIV.": "West Virginia",
    "west virginia univ.": "West Virginia",
    "wvu": "West Virginia",
    "Wvu": "West Virginia",
    "Western Mich": "Western Michigan",
    "Western Michigan": "Western Michigan",
    "western michigan": "Western Michigan",
    "western mich": "Western Michigan",
    "WESTERN MICHIGAN": "Western Michigan",
    "WESTERN MICH": "Western Michigan",
    "WILLIAM & MARY": "William & Mary",
    "William & Mary": "William & Mary",
    "COLLEGE OF WILLIAM & MARY": "William & Mary",
    "College Of William & Mary": "William & Mary",
    "william & mary": "William & Mary",
    "college of william & mary": "William & Mary",
    "WILSON": "Wilson",
    "Wilson": "Wilson",
    "wilson": "Wilson",
    "WILSON COLLEGE": "Wilson",
    "Wilson College": "Wilson",
    "wilson college": "Wilson",
    "WINONA STATE": "Winona State",
    "Winona State": "Winona State",
    "winona state": "Winona State",
    "Yale": "Yale",
    "yale": "Yale",
    "YALE": "Yale"
}


from .RtnSingleTeamYear import RtnSingleTeamYear
from .src import validate_input, get_extra_cols, SCHEDULE_COLS, RESULTS_COLS, IND_RESULTS_COLS, EVENTS, ROSTER_COLS
import pandas as pd


def save(df, filename):
    df.to_csv(filename, index=False)


def all_teams(year):
    rtn = RtnSingleTeamYear(year=year, team_name=None)
    return list(rtn.get_team_mapping().keys())


def roster(year, teams, include_hometowns=False, include_class=False, include_events=False, verbose=False):
    teams = validate_input(teams)

    all_rosters = []
    for team in teams:
        if verbose:
            print(f'Getting roster for {team}                              ', end='\r' if team != teams[-1] else None)
        rtn = RtnSingleTeamYear(year=year, team_name=team)
        res = rtn.get_roster(include_hometowns=include_hometowns, include_class=include_class,
                             include_events=include_events)
        if verbose and len(res) == 0:
            print(f'\tNo roster found for {team}')
        all_rosters.append(res)

    extra_cols = get_extra_cols(include_hometowns=include_hometowns, include_class=include_class, include_events=include_events)
    return pd.concat(all_rosters)[ROSTER_COLS + extra_cols]


def schedule(year, teams, verbose=False):
    teams = validate_input(teams)

    all_schedules = []
    for team in teams:
        if verbose:
            print(f'Getting schedule for {team}                              ', end='\r' if team != teams[-1] else None)
        rtn = RtnSingleTeamYear(year=year, team_name=team)
        res = rtn.get_schedule()
        if verbose and len(res) == 0:
            print(f'\tNo schedule found for {team}')
        all_schedules.append(res)

    return pd.concat(all_schedules)[SCHEDULE_COLS]


def team_results(year, teams, method='team_consistency', force_update=False, verbose=False):
    teams = validate_input(teams)

    all_results = []
    for team in teams:
        if verbose:
            print(f'Getting schedule and results for {team}                              ', end='\r' if team != teams[-1] else None)
        rtn = RtnSingleTeamYear(year=year, team_name=team)
        res = rtn.get_team_scores(method=method,force_update=force_update)
        if verbose and len(res) == 0:
            print(f'\tNo schedule and results found for {team}')
        all_results.append(res)

    return pd.concat(all_results)[SCHEDULE_COLS + RESULTS_COLS]


def individual_results(year, teams, method='by_meet', force_update=False, verbose=False):
    teams = validate_input(teams)

    all_scores = []
    for team in teams:
        if verbose:
            print(f'Getting scores for {team}                              ', end='\r' if team != teams[-1] else None)
        rtn = RtnSingleTeamYear(year=year, team_name=team)
        res = rtn.get_individual_scores(method=method, force_update=force_update)
        if verbose and len(res) == 0:
            print(f'\tNo scores found for {team}')
        all_scores.append(res)

    return pd.concat(all_scores)[SCHEDULE_COLS + IND_RESULTS_COLS]


def individual_nqs(year, teams, verbose=False):
    teams = validate_input(teams)

    all_nqs = []
    for team in teams:
        if verbose:
            print(f'Getting individual NQS for {team}                              ', end='\r' if team != teams[-1] else None)
        rtn = RtnSingleTeamYear(year=year, team_name=team)
        res = rtn.get_individual_nqs()
        if verbose and len(res) == 0:
            print(f'\tNo individual NQS found for {team}')
        all_nqs.append(res)

    return pd.concat(all_nqs)[ROSTER_COLS + EVENTS] # + ['AA']]


def rankings(year, team_vs_ind='team', event='AA', week=None):
    rtn = RtnSingleTeamYear(year=year, team_name=None)
    return rtn.get_overall_rankings(team_vs_ind=team_vs_ind, event=event, week=week)
